<?php

namespace App\Http\Controllers\Tixstock;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;

use App\Performers;
use App\Venues;
use App\Events;
use App\EventCategory;
use App\VenueCategory;
use App\VenueSection;
use App\Sources;
use App\TixstockLog;
use App\Http\Controllers\API\StubhubController as StubhubApi;
use App\Categories;
use App\SubCategories;
class EventsController extends Controller
{
    //
    public function addEvent(Request $request) {
        
        $addedEvents = [];
        $updatedEvents = [];
        $errorEvents = [];
        $sources=Sources::where('name','Tixstock')->first();
        $api = Http::withToken($sources->token);
        if (!$request->filled('event_id')) {
            return response()->json(['success' => false, 'errors' => ['event_id is required.'] ]);
        }
        
        $eventIds=explode(',',$request->event_id);
      
        if(!empty($eventIds)){
            foreach($eventIds as $eventId){
                $event = Events::with(['venue', 'eventcategory','venue.category','venue.category.section' ,'performer1', 'performer2'])->where('id', $eventId)->first();
                
                if(empty($event->venue->address_line_1)){
                    $stubhubApi = new StubhubApi();
                    $venue = $stubhubApi->getVenue($event->venue->tixstock_id);
                    if(isset($venue['venues'][0]) && !empty($venue['venues'][0])){
                        $event->venue->address_line_1 = $venue['venues'][0]['address']['address1'];
                        $event->venue->save();
                    }
                }
        
                if ($event) {
                        $form_data = [];
                        $form_data['id'] = $event->external_id;
                        $form_data['name'] = $event->name;
                        $form_data['datetime'] = $event->event_date_utc;
                        $form_data['currency'] = "EUR";
                        $form_data['use_tixstock_venue_details'] = 0;
                        $form_data['venue']['id'] = (string)$event->venue->id;
                        $form_data['venue']['name'] = $event->venue->name;
                        $form_data['venue']['address_line_1'] = isset($event->venue->address_line_1) && !empty($event->venue->address_line_1) ? $event->venue->address_line_1: '';
                        $form_data['venue']['address_line_2'] = $event->venue->address_line_2;
                        $form_data['venue']['city'] = $event->venue->city;
                        $form_data['venue']['country_code'] = $event->venue->country;
                        $form_data['venue']['postcode'] = $event->venue->post_code;
                        $venue_details=VenueCategory::with('sections')->where('venue_id',$event->venue->id)->get(['id','name'])->toArray();
                        $form_data['venue_details']=$venue_details;
                        $form_data['performers'][0]['id'] = ($event->performer1 !=null) ? $event->performer1->id : '';
                        $form_data['performers'][0]['name'] = ($event->performer1 !=null) ? $event->performer1->name : '';
                      
                        //get mapped category
                        $checkinParentCategory = Categories::where('name',$event->eventcategory->primary_category)->orWhere('parent',$event->eventcategory->primary_caetegory)->first();
                        if($checkinParentCategory){
                            if(!empty($checkinParentCategory->parent)){
                                $form_data['category'] = $checkinParentCategory->parent;
                            }else{
                                $form_data['category'] = "No Category";
                            }
                        }
                        else{
                            $checkinSubCategory = SubCategories::with('parentCategory')->where('name',$event->eventcategory->primary_category)->first();
                            if($checkinSubCategory){
                                if(!empty($checkinSubCategory->parentCategory->parent)){
                                    $form_data['category'] = $checkinSubCategory->parentCategory->parent;
                                }else{
                                     $form_data['category'] = "No Category";
                                }
                                
                                
                            }else{
                                $form_data['category'] = "No Category";
                            }
                        }
                        
                        // $form_data['category']=!empty($event->eventcategory->secondary_category) && $event->eventcategory->secondary_category != NULL ? $event->eventcategory->secondary_category : $event->eventcategory->primary_category ;
                        $form_data['status'] = "Active";
                        
                        if (!empty($event->performer2)) {
                            $form_data['performers'][1]['id'] = $event->performer2->id;
                            $form_data['performers'][1]['name'] = $event->performer2->name;
                        }
                  
                    if($event->update_requires == 0){
                        $response = $api->post(env('TIXSTOCK_URL').'/events/add/'.$event->external_id, $form_data);
                        $response_arr = $response->json();
                        
                        if(!empty($response_arr['data'])){
                            $event->mapped_status=1;
                            $event->save();
                            //store success log into database
                            $tixstockLog=['event_id'=>$event->id,'source_id'=>$sources->id,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'form_data'=>json_encode($form_data)];
                            TixstockLog::create($tixstockLog);
                            $addedEvents [] = $event->external_id;
                            
                            //send using stubhub token
                                $sources=Sources::where('name','Stubhub')->first();
                                $api = Http::withToken($sources->token);
                                $response = $api->post(env('TIXSTOCK_URL').'/events/add/'.$event->external_id, $form_data);
                                $response_arr = $response->json();
                                if(!empty($response_arr['data'])){
                                    //store success log into database
                                    $tixstockLog=['event_id'=>$event->id,'source_id'=>$sources->id,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'form_data'=>json_encode($form_data)];
                                    TixstockLog::create($tixstockLog);
                                }else{
                                    //store error log into database
                                    $tixstockLog=['event_id'=>$event->id,'source_id'=>$sources->id,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'status'=>1,'form_data'=>json_encode($form_data)];
                                    TixstockLog::create($tixstockLog);
                                }
                        }else{
                            //store error log in to database
                            $tixstockLog=['event_id'=>$event->id,'source_id'=>$sources->id,'type'=>'Move to tixstock:add','message'=>json_encode($response->json()),'status'=>1,'form_data'=>json_encode($form_data)];
                            TixstockLog::create($tixstockLog);
                            $errorEvents[] = $event->external_id;
                        }
                    }
                    if($event->update_requires == 1){
                         
                         $response = $api->post(env('TIXSTOCK_URL').'/events/update/'.$event->external_id, $form_data);
                         $response_arr = $response->json();
                         
                         if(!empty($response_arr['data'])){
                             //store log in to database
                             $tixstockLog=['event_id'=>$event->id,'source_id'=>$sources->id,'type'=>'Move to tixstock:update','message'=>json_encode($response->json()),'form_data'=>json_encode($form_data)];
                          
                             TixstockLog::create($tixstockLog);
                             $updatedEvents[] = $event->external_id;
                         }else{
                             //store error log into database
                             $tixstockLog=['event_id'=>$event->id,'source_id'=>$sources->id,'type'=>'Move to tixstock:update','message'=>json_encode($response->json()),'status'=>1,'form_data'=>json_encode($form_data)];
                             TixstockLog::create($tixstockLog);
                             $errorEvents[] =$event->external_id; 
                         }
                         $event->update_requires = 0;
                         $event->save();
                    }
                }
            }
            $successmessage = "";
            $errormessage = "";
            if(!empty($addedEvents)){
                $successmessage  .= "(".implode(',',$addedEvents).") Events Added Successfully. ";
            }
            if(!empty($updatedEvents)){
                $successmessage  .= "(".implode(',',$updatedEvents).") Events Updated Successfully. ";
            }
            if(!empty($errorEvents)){
                $errormessage  .= "(".implode(',',$errorEvents).") Something went wrong with mentioned events.";
            }
            return response()->json(['success'=>true,"successmsg"=>$successmessage,"errormsg"=>$errormessage]);
        }else{
            return response()->json(['success'=>false,"msg"=>"events not found."]);
        }
    }


    public function getEvents($page=null){

        $sources=Sources::where('name','Stubhub')->first();
        if($page==null){
            $page=33;
        }
        echo $page;
        $perPage=50;
        $url="https://sandbox-api.tixstock.com/v1/events/get?mapped_status=false&per_page=50&page=".$page;
        $response=$this->fetchData($url,$sources->token);
        dd($response);
        if(!empty($response['data'])){
            foreach($response['data'] as $events)
            {
                //check venue details insert if needed
                $checkVenue=Venues::where('name',$events['venue']['name'])
                                    ->where('city',$events['venue']['city'])
                                    ->where('post_code',$events['venue']['postcode']);
                                if(!empty($events['venue']['id'])){
                                    $checkVenue=$checkVenue->where('tixstock_id',$events['venue']['id']);
                                }
                $checkVenue=$checkVenue->first();
                if(!$checkVenue){
                    $venueArray=[
                                'tixstock_id'=>$events['venue']['id'],
                                'name'=>$events['venue']['name'],
                                'address_line_1'=>$events['venue']['address_line_1'],
                                'address_line_2'=>$events['venue']['address_line_2'],
                                'city'=>$events['venue']['city'],
                                'state'=>$events['venue']['state'],
                                'post_code'=>$events['venue']['postcode'],
                                'country'=>$events['venue']['country_code'],
                                'source_id'=>$sources->id
                    ];
                    $venue=Venues::create($venueArray);
                    $venueId=$venue->id;
                }else{
                    $venueId=$checkVenue->id;
                }
                //check venue section and category
                if(!empty($events['venue_details'])){
                    foreach($events['venue_details'] as $venueDetail){
                        //check venue category
                        $checkCategory = VenueCategory::where('name',$venueDetail['name'])
                                                        ->where('venue_id',$venueId)
                                                        ->where('source_id',$sources->id)
                                                        ->first();
                            if(!$checkCategory){
                                $categoryArray=[
                                                'name'=>$venueDetail['name'],
                                                'venue_id'=>$venueId,
                                                'venue_category'=>$venueDetail['name'],
                                                'source_id'=>$sources->id
                                            ];
                                $category = VenueCategory::create($categoryArray);
                                $categoryId = $category->id;
                            }else{
                                $categoryId = $checkCategory->id;
                            }
                            //check section
                            foreach($venueDetail['sections'] as $section){
                                    $checkSection=VenueSection::where('name',$section['name'])->where('source_id',$sources->id)->first();
                                    if(!$checkSection){
                                        $sectionArray = [
                                                            'name'=>$section['name'],
                                                            'category_id'=>$categoryId,
                                                            'source_id'=>$sources->id
                                                        ];
                                        $section = VenueSection::create($sectionArray);
                                    }
                            }
                    }
                }
                
                //check performers 
                $count=1;
                $performer_1_id=null;
                $performer_2_id=null;
                foreach($events['performers'] as $performer){
                    $checkPerformer=Performers::where('name',$performer['name']);
                    if(!empty($performer['id'])){
                        $checkPerformer=$checkPerformer->where('external_id',$performer['id']);
                    }
                                               
                    $checkPerformer=$checkPerformer->where('source_id',$sources->id)->first();
                    if(!$checkPerformer){
                        $performerArray=[
                                            'name'=>$performer['name'],
                                            'external_id'=>$performer['id'],
                                            'source_id'=>$sources->id,
                        ];
                        $performer = Performers::create($performerArray);
                        if($count==1){
                            $performer_1_id = $performer->id;
                        }
                        else{
                            $performer_2_id= $performer->id;
                        }
                    }else{
                        if($count==1){
                            $performer_1_id=$checkPerformer->id;
                        }else{
                            $performer_2_id = $checkPerformer->id;
                        }
                    }
                    $count++;
                }

                //check events 
                $event_name = isset($events['name']) ? $events['name'] : '';
                $event_date = isset($events['datetime']) ? date ('Y-m-d H:i:s',strtotime($events['datetime'])) : '';
                $event_date_utc = isset($events['datetime']) ? $events['datetime'] : '';

                $eventArray['search_type'] = $events['category']['name'];
                $eventArray['name'] = str_replace("'", "",$event_name);
                $eventArray['performer_1_id'] = $performer_1_id;
                $eventArray['performer_2_id'] = $performer_2_id;
                $eventArray['event_date'] = $event_date;
                $eventArray['event_date_utc'] = $event_date_utc;
                $eventArray['venue_id'] = $venueId;
                $eventArray['source_id']=$sources->id;
                $eventArray['mapped_status']=true;
                $checkEvents=Events::where($eventArray)->first();
               
                if(!$checkEvents){
                    $eventArray['external_id'] = isset($events['id']) ? $events['id'] : '';
                    $newevent = Events::create($eventArray);
                    $eventId = $newevent->id;
                }else{
                    $eventId = $checkEvents->id;
                }

                //check event category
                $checkEventCategory = EventCategory::where('primary_category',$events['category']['name'])->where('event_id',$eventId)->where('source_id',$sources->id)->first();
                if(!$checkEventCategory){
                    $eventCategoryArray=['primary_category'=>$events['category']['name'],'event_id'=>$eventId,'venue_id'=>$venueId,'source_id'=>$sources->id];
                    EventCategory::create($eventCategoryArray);
                }
            }
        $page++;
        $this->getEvents($page);
        }else{
            echo "all data received";
            exit;
        }
        
    }

    public function fetchData($url,$token)
    {

        $curl = curl_init();

        curl_setopt_array($curl, array(
        CURLOPT_URL => $url,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 30,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => "GET",
        CURLOPT_HTTPHEADER => array(
            "authorization: Bearer ".$token,
            "cache-control: no-cache",
            "content-type: application/x-www-form-urlencoded",
            "postman-token: 2dd383dd-1b1d-af80-96c9-c8ee7195cff0"
        ),
        ));

        $response = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);

        if ($err) {
            return $err;
        } else {
            return json_decode($response,true);
        }
    }
}
