<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Http;

use App\Performers;
use App\Venues;
use App\Events;
use App\EventCategory;
use App\VenueCategory;
use App\VenueSection;
use App\Sources;
use Illuminate\Support\Facades\Log;
class FetchTixstockData extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'fetch:tixstockdata {page}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Command description';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        $page=$this->arguments()['page'];
        $this->processData($page);
    }
    
    public function processData($page){
        
        $sources=Sources::where('name','Stubhub')->first();
        $url="https://sandbox-api.tixstock.com/v1/events/get?mapped_status=true&per_page=50&page=".$page;
        $response=$this->fetchData($url,$sources->token);
        
        Log::info("page" .$page." fetched");
        
        if(!empty($response['data'])){
            foreach($response['data'] as $events)
            {
                //check venue details insert if needed
                $venueArray = [];
                $venueArray = [
                        'name'=>isset($events['venue']['name']) && !empty($events['venue']['name']) ? $events['venue']['name'] : NULL,
                        'city'=>isset($events['venue']['city']) && !empty($events['venue']['city']) ? $events['venue']['city'] : "",
                        'address_line_1'=>isset($events['venue']['address_line_1']) && !empty($events['venue']['address_line_1']) ? $events['venue']['address_line_1'] : NULL,
                        'state'=>isset($events['venue']['state']) && !empty($events['venue']['state']) ? $events['venue']['state'] : NULL,
                        'post_code'=>isset($events['venue']['postcode']) && !empty($events['venue']['state']) ? $events['venue']['postcode'] : NULL,
                        'country'=>isset($events['venue']['country_code']) && !empty($events['venue']['country_code']) ? $events['venue']['country_code'] : NULL,
                    ];
               
                
                $checkVenue=Venues::where($venueArray)->first();
              
                if(!$checkVenue){
                    $venueArray['tixstock_id'] = $events['venue']['id'];
                    $venueArray['address_line_2'] = isset($events['venue']['address_line_2']) && !empty($events['venue']['address_line_2']) ? $events['venue']['address_line_2'] : NULL;
                    $venueArray['source_id'] = $sources->id;
                    $venue=Venues::create($venueArray);
                    $venueId=$venue->id;
                }else{
                    $checkVenue->tixstock_id = $events['venue']['id'];
                    $checkVenue->save();
                    
                    $venueId=$checkVenue->id;
                }
                
                //check venue section and category
                if(!empty($events['venue_details'])){
                    foreach($events['venue_details'] as $venueDetail){
                        //check venue category
                        $checkCategory = VenueCategory::where('name',$venueDetail['name'])
                                                        ->where('venue_id',$venueId)
                                                        ->where('source_id',$sources->id)
                                                        ->first();
                            if(!$checkCategory){
                                $categoryArray=[
                                                'name'=>$venueDetail['name'],
                                                'venue_id'=>$venueId,
                                                'venue_category'=>$venueDetail['name'],
                                                'source_id'=>$sources->id
                                            ];
                                $category = VenueCategory::create($categoryArray);
                                $categoryId = $category->id;
                            }else{
                                $categoryId = $checkCategory->id;
                            }
                            //check section
                            foreach($venueDetail['sections'] as $section){
                                    $checkSection=VenueSection::where('name',$section['name'])->where('category_id',$categoryId)->where('source_id',$sources->id)->first();
                                    if(!$checkSection){
                                        $sectionArray = [
                                                            'name'=>$section['name'],
                                                            'category_id'=>$categoryId,
                                                            'source_id'=>$sources->id
                                                        ];
                                        $section = VenueSection::create($sectionArray);
                                    }
                            }
                    }
                }
                
                //check performers 
                $count=1;
                $performer_1_id=null;
                $performer_2_id=null;
                foreach($events['performers'] as $performer){
                    $checkPerformer=Performers::where('name',$performer['name']);
                    if(!empty($performer['id'])){
                        $checkPerformer=$checkPerformer->where('external_id',$performer['id']);
                    }
                                               
                    $checkPerformer=$checkPerformer->where('source_id',$sources->id)->first();
                    if(!$checkPerformer){
                        $performerArray=[
                                            'name'=>$performer['name'],
                                            'external_id'=>$performer['id'],
                                            'source_id'=>$sources->id,
                        ];
                        $performer = Performers::create($performerArray);
                        if($count==1){
                            $performer_1_id = $performer->id;
                        }
                        else{
                            $performer_2_id= $performer->id;
                        }
                    }else{
                        if($count==1){
                            $performer_1_id=$checkPerformer->id;
                        }else{
                            $performer_2_id = $checkPerformer->id;
                        }
                    }
                    $count++;
                }

                //check events 
                $event_name = isset($events['name']) ? $events['name'] : '';
                $event_date = isset($events['datetime']) ? date ('Y-m-d H:i:s',strtotime($events['datetime'])) : '';
                $event_date_utc = isset($events['datetime']) ? $events['datetime'] : '';

                $eventArray['search_type'] = isset($events['category']['name']) && !empty($events['category']['name']) ? $events['category']['name'] : '';
                $eventArray['name'] = str_replace("'", "",$event_name);
                $eventArray['performer_1_id'] = $performer_1_id;
                $eventArray['performer_2_id'] = $performer_2_id;
                $eventArray['event_date'] = $event_date;
                $eventArray['event_date_utc'] = $event_date_utc;
                $eventArray['venue_id'] = $venueId;
                $eventArray['source_id']=$sources->id;
                $eventArray['mapped_status']=true;
                $eventArray['status'] = $events['status'];
                $eventArray['use_tixstock_venue_details'] = isset($events['use_tixstock_venue_details']) && !empty($events['use_tixstock_venue_details']) && $events['use_tixstock_venue_details'] == "true" ? 1 : 0 ;
                $eventArray['currency_code']= isset($events['currency']) ? $events['currency'] : '';
                $checkEvents=Events::where($eventArray)->first();
               
                if(!$checkEvents){
                    $eventArray['external_id'] = isset($events['id']) && !empty($events['id']) ? $events['id'] : '';
                    $newevent = Events::create($eventArray);
                    $eventId = $newevent->id;
                }else{
                    
                    $eventId = $checkEvents->id;
                    $checkEvents->status=$events['status'];
                    $checkEvents->use_tixstock_venue_details = $events['use_tixstock_venue_details'];
                    $checkEvents->save();
                }

                //check event category
                if($events['category'] != null){
                    $checkEventCategory = EventCategory::where('primary_category',$events['category']['name'])->where('event_id',$eventId)->where('source_id',$sources->id)->first();
                    if(!$checkEventCategory){
                        $eventCategoryArray=['primary_category'=>$events['category']['name'],'event_id'=>$eventId,'venue_id'=>$venueId,'source_id'=>$sources->id];
                        EventCategory::create($eventCategoryArray);
                    }
                }
            }
        $page++;
        $this->processData($page);
        }else{
            echo "all data received";
            exit;
        }
    }
    
    public function fetchData($url,$token)
    {

        $curl = curl_init();

        curl_setopt_array($curl, array(
        CURLOPT_URL => $url,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_ENCODING => "",
        CURLOPT_MAXREDIRS => 10,
        CURLOPT_TIMEOUT => 30,
        CURLOPT_HTTP_VERSION => CURL_HTTP_VERSION_1_1,
        CURLOPT_CUSTOMREQUEST => "GET",
        CURLOPT_HTTPHEADER => array(
            "authorization: Bearer ".$token,
            "cache-control: no-cache",
            "content-type: application/x-www-form-urlencoded",
            "postman-token: 2dd383dd-1b1d-af80-96c9-c8ee7195cff0"
        ),
        ));

        $response = curl_exec($curl);
        $err = curl_error($curl);

        curl_close($curl);

        if ($err) {
            return $err;
        } else {
            return json_decode($response,true);
        }
    }
}
