<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use DataTables;
use App\Venues;
use App\VenueCategory;
use App\VenueSection;
class VenueController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        //
        return view('admin.venue.index');
    }

    public function data(){
        $venues=Venues::get();
        
        return Datatables::of($venues)
        ->rawColumns(['action','category_section','edit'])
        ->editColumn('address', function ($venues) {
            return $venues->address_line_1.' '.$venues->address_line_2;
        })
        ->editColumn('postcode', function ($venues) {
            return $venues->post_code;
        })
        ->addColumn('category_section', function ($venues) {
            $html='<button type="button" id="'.$venues->id.'" data-source="'.$venues->source_id.'" class="ladda-button btn btn-danger waves-effect waves-light category_section_detail_btn" dir="ltr" data-style="expand-right" title="View Detail" data-original-title="View Detail" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" data-tippy=""  ><i class="mdi mdi-eye"></i></button>';
            return $html;
        })
         ->addColumn('edit', function (Venues $venues) {
            $html = '<a href="'.route('venue.edit',$venues->id).' " class="action-icon edit_icon"> <i class="mdi mdi-square-edit-outline"></i></a>';
            return $html;
        })
        ->addColumn('action', function (Venues $venues) {

            $html = '<button id="'.$venues->id.'" title="Add Category" tabindex="0" data-plugin="tippy" data-tippy-animation="scale" data-tippy-inertia="true" data-tippy-duration="[600, 300]" data-tippy-arrow="true" class="btn btn-blue mb-2"><i class="mdi mdi-plus-circle"></i></a>';
            $action = $html;
            return $action;
        })
        ->make(true);
    }

    public function getVenueCategorySectionData(Request $request){
        
        // $venueCategorySection = VenueCategory::with('section')->where('venue_id',$request->id)->get();
        $venueId=$request->id;
        $category = VenueCategory::with('section')->where('venue_id',$request->id)->where('source_id',$request->source_id)->get();
        $tixstockCategory = VenueCategory::with('section')->where('venue_id',$request->id)->where('source_id',0)->get();
        $html = view('admin.events.ajax.event_venue_section',compact('category','tixstockCategory','venueId'))->render();
        //$html= view('admin.category.ajax.category_section',compact('venueCategorySection'))->render();
       

        return response()->json(['html'=>$html,'id'=>$request->id]);

    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
        return view('admin.venue.create');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {

        //
        $rules = [
            'name'         => 'required',
            'city'         => 'required',
            // 'postcode'     => 'required',
            'address'      => 'required',
            'state'        => 'required',
            'country'      => 'required'
        
        ];
        $this->validate($request, $rules);

        /*        $category = strip_tags(trim($request->category));
        $category = explode('\n',$category);
        $category = array_filter($category,'trim');
        $sendCategoryId = [];
        foreach($category as $c){
            $check = VenueCategory::where('name',trim($c))->first();
            if(!$check){
                $venueCategory=VenueCategory::create(['name'=>$c]);
                $venueCategoryId = $venueCategory->id;
            }
            else{
                $venueCategoryId = $check->id;
            }
            $sendCategoryId[]= $venueCategoryId;
        }

        if(!empty($sendCategoryId)){
            $sendCategoryId = implode(',',array_unique($sendCategoryId));
        }

        $sendSectionId = [];
        $section = trim($request->section);
        $section = explode('\n',$section);
        $section = array_filter($section,'trim');

        foreach($section as $s){
            
            $venueSection = VenueSection::where('name',$s)->first();
            if(!$venueSection){
                $venueSection = VenueSection::create(['name'=>$s]);
                $venueSectionId = $venueSection->id;
            }else{
                $venueSectionId = $venueSection->id;
            }

            $sendSectionId[] = $venueSectionId;
        }

        if(!empty($sendSectionId)){
            $sendSectionId = implode(',',array_unique($sendSectionId));
        }
        */
        $input= [
            'name'=>$request->name,
            'city'=>$request->city,
            'state'=>$request->state,
            'address_line_1'=>$request->address,
            'country'=>$request->country,
            'postcode'=>$request->postcode,
        ];

        Venues::create($input);
        return response()->json(['success'=>true,'msg'=>'Venue Created Successfully']);
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
        $venue = Venues::with(['category','category.section'])->where('id',$id)->first();
        return view('admin.venue.edit',compact('venue'));
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {

        //
        $rules = [
            // 'category'     => 'required',
            // 'section'      => 'required',
            'name'         => 'required',
            'city'         => 'required',
            'postcode'     => 'required',
            'address'      => 'required',
            'state'        => 'required',
            'country'      => 'required'
        
        ];
        $this->validate($request, $rules);

        //Check category 
        /*
        $category = strip_tags(trim($request->category));
        $category = explode('\n',$category);
        $category = array_filter($category,'trim');
        $sendCategoryId = [];
        foreach($category as $c){
            $check = VenueCategory::where('name',trim($c))->first();
            if(!$check){
                $venueCategory=VenueCategory::create(['name'=>$c]);
                $venueCategoryId = $venueCategory->id;
            }
            else{
                $venueCategoryId = $check->id;
            }
            $sendCategoryId[]= $venueCategoryId;
        }

        if(!empty($sendCategoryId)){
            $sendCategoryId = implode(',',array_unique($sendCategoryId));
        }

        //check section 
        $sendSectionId = [];
        $section = trim($request->section);
        $section = explode('\n',$section);
        $section = array_filter($section,'trim');

        foreach($section as $s){
            
            $venueSection = VenueSection::where('name',$s)->first();
            if(!$venueSection){
                $venueSection = VenueSection::create(['name'=>$s]);
                $venueSectionId = $venueSection->id;
            }else{
                $venueSectionId = $venueSection->id;
            }

            $sendSectionId[] = $venueSectionId;
        }

        if(!empty($sendSectionId)){
            $sendSectionId = implode(',',array_unique($sendSectionId));
        }
        */
        $input= [
            'name'=>$request->name,
            'city'=>$request->city,
            'state'=>$request->state,
            'address_line_1'=>$request->address,
            'country'=>$request->country,
            'post_code'=>$request->postcode,
        ];

        Venues::where('id',$id)->update($input);
        return response()->json(['success'=>true,'msg'=>'Venue Created Successfully']);
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }

    public function deleteSection(Request $request){
        
        VenueSection::where('id',$request->id)->delete();
        $response=['success'=>true,'msg'=>"Section deleted successfully."];
        return response()->json($response);
    }

    public function deleteCategory(Request $request){
        
        $category=Venuecategory::where('id',$request->id)->first();
        $category->section()->delete();
        $category->delete();
        
        $response=['success'=>true,'msg'=>"Category deleted successfully."];
        return response()->json($response);
    }
}
