<?php 
namespace App\Traits;
use App\Performers;
use App\Venues;
use App\Events;
use App\EventCategory;
use App\VenueCategory;
use App\VenueSection;
use App\Sources;
use App\Logs;
use Illuminate\Support\Facades\Log;
use App\Traits\ApiTrait;
use App\Http\Controllers\API\StubhubController as StubhubApi;
trait StubhubTrait{ 
    use ApiTrait;

    public function fetchStubhubData($resource,$slug){
       
        if(in_array($slug,['Sports','Music','UpdateData'])){
            $this->fetchStubhubEvents($resource,$slug);
        }
        
        if(in_array($slug,['Zones'])){
            //fetch zones
            $this->fetchStuhubhubEventsZones($resource,$slug);
        }
    }

    public function fetchStubhubEvents($resource,$slug){
        
        $this->processData($resource,$slug,null);
      
    }
    
    public function processData($resource,$slug,$start=null){
        
        date_default_timezone_set('Europe/London');
        $date="";
        $countResults = 0;
        $totalResults = 0;
        $createdEvents=[];
        $updatedEvents=[];
        $day = "";
        
        
        if($slug=="UpdateData"){
            $days = 10;
            $date= date('Y-m-d', strtotime(' - '.$days.' days'));
        }
        
        if($date==""){
            $date=date('Y-m-d');
            $start = Events::where('search_type','Music')->where('mapped_status',0)->get()->count();
            Log::info($start." fetched on ".$date);
            $url=env('STUBHUB_URL')."?categoryName=".$slug."&rows=500&start=".$start;
        }else{
            if($start == null){
                $start=0;
            }
            $url="https://api.stubcloudprod.com/sellers/search/bulk/events/v3?dateLastModified=".$date."&rows=500&start=".$start;
            $stubhubapi = new StubhubApi();
            $url=env('STUBHUB_API_URL')."".$stubhubapi::URL_GET_EVENT."?dateLastModified=".$date."&rows=500&start=".$start;
        }
        //create stuhub  request to fetch events
        $response=$this->fetchData($url);
        
        $totalResults=isset($response['numFound']) && !empty($response['numFound']) ? $response['numFound'] : 0;
        
        $log = Log::channel('cron');
        $log->info( "start: ".$start." of ".$totalResults." event");
        
        if(isset($response['events']) && count($response['events']) >= 1){
            foreach ($response['events'] as $key => $value) {
              $log->info( "Processing event: " .$value['id']);
                $event_category=array();
                $cat_arr=array();
                $event_arr = array();

                $performer_1_id = NULL;
                $performer_2_id = NULL;
                $venue_id = 0;
                $event_id = 0;
                
                
                //check country code if its not in europe skip it
                if(isset(Events::COUNTRY_CODES[$value['venue']['country']]) === false) {
                    echo 'Skipped event invalid country ' . $value['venue']['country'] . "\r\n";
                    $log->info($value['id']." event skipped.");
                    $log->info("=============================================");
                    continue;
                }
                
                
                //get currency code
                $currencyCode="";
                $eventWithCurrency = $stubhubapi->getEvent($value['id']);
                if(!empty($eventWithCurrency) && !empty($eventWithCurrency['events'][0])){
                    $currencyCode = $eventWithCurrency['events'][0]['currencyCode'];
                }
               
                //performers data
                if(isset($value['performers']) && !empty($value['performers'])){
                    foreach($value['performers'] as $pk =>$performer){
                        $performer_arr=array();

                        //performer name
                        $performer_name = isset($performer['name']) ? $performer['name'] : '';
                        $performer_arr['name'] = str_replace("'", "",$performer_name);
                        $performer_arr['name_variation'] = str_replace("'", "",$performer_arr['name']);
                        $performer_arr['source_id']=$resource->id;
                        $performer_arr['external_id']=isset($performer['id']) && !empty($performer['id']) ? $performer['id'] : '';
                        
                        $checkPerformer = Performers::where('name',$performer_arr['name'])->where('source_id',$resource->id)->first();
                        if(!$checkPerformer){   
                            $performer = Performers::create($performer_arr);
                            $performerId = $performer->id;
                            $log->info("Performer Id:".$performerId." created for event ".$value['id']);
                            
                        }
                        else{
                            if(!empty($checkPerformer->external_id) && !empty($perfomrer_arr['external_id']) && $checkPerformer->external_id != $performer_arr['external_id']){
                                $checkPerformer->external_id = $perfomrer_arr['external_id'] ;
                                $checkPerformer->save();
                                $log->info("Performer External id :".$perfomrer_arr['external_id'] ." saved for event ".$value['id']);
                            }
                            $performerId = $checkPerformer->id;
                        }

                        if($pk == 0){
                            $performer_1_id = $performerId;
                        }
                        if(count($value['performers'])>=2 && $pk == 1){
                            $performer_2_id = $performerId;
                        }
                    }
                }

                //venues data
                if(isset($value['venue']) && !empty($value['venue'])){
                   
                    $venue_arr = array();
                    $venue_name = isset($value['venue']['name']) ? $value['venue']['name'] : NULL;
                    $city = isset($value['venue']['city']) ? $value['venue']['city'] : NULL;
                    $state = isset($value['venue']['state']) ? $value['venue']['state'] : NULL;
                    $postalCode = isset($value['venue']['postalCode']) ? $value['venue']['postalCode'] : NULL;
                    $country = isset($value['venue']['country']) ? $value['venue']['country'] : NULL;

                    $venue_arr['name'] = str_replace("'", "",$venue_name);
                    $venue_arr['city'] = str_replace("'", "",$city);
                    $venue_arr['state'] = str_replace("'", "",$state);
                    $venue_arr['post_code'] = str_replace("'", "",$postalCode);
                    $venue_arr['country'] = str_replace("'", "",$country);
                    $venue_arr['source_id']=$resource->id;
                    
                    $venueDetails = $stubhubapi->getVenue($value['venue']['id']);
                    if(isset($venueDetails['venues'][0]) && !empty($venueDetails['venues'][0])){
                       $venue_arr['address_line_1'] = $venueDetails['venues'][0]['address']['address1'];
                    }
                   //check venue 
                    $checkVenue = Venues::where($venue_arr)->first();
                    if(!$checkVenue){
                        $venue_arr['tixstock_id']=$value['venue']['id'];
                         $checkVenueWithId=Venues::where($venue_arr)->first();
                         if(!$checkVenueWithId){
                          $venue = Venues::create($venue_arr);
                          $venue_id=$venue->id;
                          
                          $log->info("Venue Id:".$venue_id." created for event ".$value['id']);
                         }else{
                             $venue_id=$checkVenueWithId->id; 
                         }
                    }else{
                        $venue_id=$checkVenue->id;
                        if($checkVenue->tixstock_id != $value['venue']['id']){
                            $checkVenue->tixstock_id=$value['venue']['id'];
                            $checkVenue->save();
                            $log->info("Venue External id :".$value['venue']['id'] ." saved for event ".$value['id']);
                        }
                    }
                }

                //event data
                
                $newEvent = 0;
                $new_event_id = '';
                $event_name = isset($value['name']) ? $value['name'] : '';
                $event_date = isset($value['eventDateLocal']) ? date('Y-m-d H:i:s',strtotime($value['eventDateLocal'])) : '';
                $event_date_utc = isset($value['eventDateUTC']) ? $value['eventDateUTC'] : '';

                if(isset($value['ancestors']['categories'][0]['name']) && $value['ancestors']['categories'][0]['name'] == "Sports"){
                    $value['ancestors']['categories'][0]['name']= "Sport";
                }
                
                $event_arr['search_type'] = isset($value['ancestors']['categories'][0]['name']) && !empty($value['ancestors']['categories'][0]['name']) ? $value['ancestors']['categories'][0]['name'] : '';
                $event_arr['name'] = str_replace("'", "",$event_name);
                $event_arr['performer_1_id'] = $performer_1_id;
                $event_arr['performer_2_id'] = $performer_2_id;
                $event_arr['event_date'] = $event_date;
                $event_arr['event_date_utc'] = $event_date_utc;
                $event_arr['venue_id'] = $venue_id;
                $event_arr['source_id']=$resource->id;
                $event_arr['status'] = $value['status'];
                
                //check event external id exist or not 
                $checkExternalId= Events::where('external_id',$value['id'])->first();
                if(!$checkExternalId){
                    // if event with external id doesn't exist create event
                    $event_arr['external_id'] = isset($value['id']) ? $value['id'] : '';
                    $checkEvent=Events::where($event_arr)->first();
                    if(!$checkEvent){
                        $event_arr['url'] = isset($value['webURI']) ? $value['webURI'] : '';
                        $event_arr['status']=$value['status'];
                        $event_arr['currency_code']= isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                        $event=Events::create($event_arr);
                        $event_id=$event->id;
                        $newEvent = 1;
                        
                        $log->info("created  event with external id : ".$value['id']);
                        //craete zone and section
                        //get zone and section of event 
                        $this->fetchStuhubhubEventsZones($resource,$event->external_id);
                        $createdEvents[] = $event->external_id;
                    }else{
                        $event_id=$checkEvent->id;
                    }  
                }else{
                    
                    //check if same event exist or not 
                    $checkEventUpdate=Events::where($event_arr)->first();
                    if(!$checkEventUpdate){
                        //event is updated
                        $event_id = $checkExternalId->id;
                        $event_arr['url'] = isset($value['webURI']) ? $value['webURI'] : '';
                        $eventUpdateFlag=0;
                        
                        if($checkExternalId->mapped_status == 1){
                            $prevArray = $checkExternalId->toArray();
                           
                            foreach($event_arr as $key=>$val){
                                if($prevArray[$key] != $val){
                                    $eventUpdateFlag=1;
                                    $event_arr['prev_'.$key] = $prevArray[$key];
                                    $log->info("Updated event's ".$key." to: ".$value['id']);
                                }
                            }
                            
                        }
                        
                        if($eventUpdateFlag==1){
                            Events::where('id',$checkExternalId->id)->update(['current_version'=>0]);
                            
                            //create new event
                            $event_arr['external_id'] = $value['id'] ;
                            $event_arr['url'] = isset($value['webURI']) ? $value['webURI'] : '';
                            $event_arr['status']=$value['status'];
                            $event_arr['current_version'] = 1;
                            $event_arr['mapped_status'] = 0;
                            $event_arr['currency_code'] = isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                            $event_arr['update_requires']=1;
                            $event = Events::create($event_arr);
                            $new_event_id = $event->id;
                            $newEvent = 1;
                            $log->info('Events Updated for :'.$value['id']);
                            $updatedEvents[] = $checkExternalId->external_id;
                            $log->info("Event ".$value['id']." updated ");
                        }else{
                             $event_arr['currency_code'] = isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                             Events::where('id',$checkExternalId->id)->update($event_arr);
                        }
                        
                    }else{
                        $checkEventUpdate->currency_code = isset($currencyCode) && !empty($currencyCode) ? $currencyCode : '';
                        $checkEventUpdate->save();
                    }
                }
                
    
                //event category data
                
                if(isset($value['ancestors']) && !empty($value['ancestors'])){
                    $cat_arr['event_id'] = $event_id;
                    if(isset($value['ancestors']['categories']) && !empty($value['ancestors']['categories'])){
                        foreach($value['ancestors']['categories'] as $ck=>$cvalue){
                            
                            $cat_name = isset($cvalue['name']) ? $cvalue['name'] : '';
                            $cat_name = str_replace("'", "",$cat_name);
                            if($ck == 0){
                                $cat_arr['primary_category'] = $cat_name;
                            }
                            if(count($value['ancestors']['categories']) >= 2 && $ck == 1){
                                $cat_arr['secondary_category'] = $cat_name;
                            }
                        }
                    }
                }

                if(!array_key_exists('secondary_category',$cat_arr)){
                    if(isset($value['categoriesCollection']) && !empty($value['categoriesCollection'])){
                        if(isset($value['categoriesCollection']['categories']) && !empty($value['categoriesCollection']['categories'])){
                            foreach($value['categoriesCollection']['categories'] as $cck=>$ccvalue){
                                if($cck == 1){
                                    $cat_name = isset($ccvalue['name']) ? $ccvalue['name'] : '';
                                    $cat_arr['secondary_category'] = str_replace("'", "",$cat_name);
                                }
                            }
                        } 
                    }
                }

                if(isset($value['ancestors']['groupings']) && !empty($value['ancestors']['groupings'])){
                    foreach($value['ancestors']['groupings'] as $gk=>$gvalue){
                        $group_name = isset($gvalue['name']) ? $gvalue['name'] : '';
                        $cat_arr['sub_category'] = str_replace("'", "",$group_name);
                    }
                }
                
               
                if(!empty($new_event_id)){
                    $event_id = $new_event_id;
                    $cat_arr['event_id'] = $new_event_id;
                }
                
                $checkEventCategory=EventCategory::where('event_id',$event_id)->first();
               
                if(!$checkEventCategory){
                    $cat_arr['venue_id']=$venue_id;
                    $cat_arr['source_id']=$resource->id;
                    EventCategory::create($cat_arr);
                    $log->info("Event category created for event :".$value['id']);
                }
                
                
                if($newEvent == 1){
                    $this->fetchStuhubhubEventsZones($resource,$event->external_id);
                    
                }
                 $log->info("End event processing of event :".$value['id']);
                  $log->info("=============================================");
            }
        }else{
            $logData=['type'=>"cron",'domain'=>"Stubhub:last ".$days." days data","message"=>"No events found."];
            Logs::create($logData);
            Log::info("No events found.");
            exit;
        }

        $countResults+=count($response['events']);
        $countResults=$start+$countResults;

        if($countResults != $totalResults){
            $this->processData($resource,$slug,$countResults);
        }
        
        $logMessage='';
        if(!empty($createdEvents)){
            $logMessage.='('.implode(',',$createdEvents).") events Created.";
        }
        if(!empty($updatedEvents)){
            $logMessage.='('.implode(',',$updatedEvents).") events Updated";
        }
        Log::info("last 10 day's data updated");
        if(!empty($logMessage)){
            $logData=['type'=>'cron','domain'=>"Stubhub:last ".$days." days data","message"=>$logMessage];
            Logs::create($logData);
        }

    }
    
    public function fetchStuhubhubEventsZones($resource,$external_id){

        $event=Events::where('is_run',0)->where('external_id',$external_id)->orderBy('id','desc')->first();
        $log=Log::channel('cron');
        $log->info("Fetching ".$external_id." Event's Zone thorugh stubhub Api");
        $eventsUpdated = [];
        if($event){
            // foreach($events as $event){
                $log->info('Processing event '.$event->external_id);
                $event_id=$event->external_id;
                $id=$event->id;
                $venue_id=$event->venue_id;
                $url = "https://api.stubcloudprod.com/partners/catalog/events/v3/".$event_id."/sectionZones";
                $response=$this->fetchData($url);
                
                if(!empty($response)){
                    if(count($response['zones']) >= 1){
                        $venue_category = 0;
                        $send_category_arr = array();
                        $send_section_arr = array();
                        $getCategory = EventCategory::where('event_id',$id)->where('venue_id',$venue_id)->where('source_id',$resource->id)->first();
    
                        $flag=0;
                        if(!empty($getCategory->secondary_category)){
                            $flag=1;
                            $venue_category = $getCategory->secondary_category;
                        }
                        if($flag==0){
                            if(!empty($getCategory->sub_category)){
                                $flag=1;
                                $venue_category = $getCategory->sub_category;
                            }
                        }
                        if($flag==0){
                            if(!empty($getCategory->primary_category)){
                                $flag=1;
                                $venue_category = $getCategory->primary_category;
                            }
                        }
                        foreach ($response['zones'] as $key => $value) {
                            $cat_arr = array();
                            $category_id = 0;
                            $category_name = $value['name'];
    
                            $cat_arr['name'] = str_replace("'", "",$category_name);
                            $cat_arr['venue_id'] = $venue_id;
                           // $cat_arr['venue_category'] = $venue_category;
                            $cat_arr['source_id'] = $resource->id;
    
                            $checkCategory=VenueCategory::where($cat_arr)->first();
                            if(!$checkCategory){
                                $cat_arr['venue_category'] = $venue_category;
                                $category = VenueCategory::create($cat_arr);
                                $category_id = $category->id;
                                $log->info('Created Category'.$category_id.' for event '.$event->external_id);
                            }else{
                                $category_id = $checkCategory->id;
                            }
                            //add sections of venue 
                            foreach ($value['sections'] as $sk => $section_value) {
                                $sec_arr = array();
                                $section_name = $section_value['name'];
                                $sec_arr['name'] = str_replace("'", "",$section_name);
                                $sec_arr['category_id'] = $category_id;
                                $sec_arr['source_id'] = $resource->id;
                                
                                //check same section is available or not 
                                $checkVenueSection=VenueSection::where($sec_arr)->first();
                                if(!$checkVenueSection){
                                    $venueSection = VenueSection::create($sec_arr);
                                    $section_id = $venueSection->id;
                                     $log->info('Created section'.$section_id.' for category '.$category_id);
                                }else{
                                    $section_id = $checkVenueSection->id;
                                }
                            }
                        }
                    }
                    $update_data = array();	
                    $update_data['is_run'] = strtotime('now');
                    Events::where('id',$id)->update($update_data);
                    $eventsUpdated[] = $event->external_id;
                    
                    $log->info($event->id." Event's zone updated successfully.");
                    $log->info("==================================");
                }
            
            if(!empty($eventsUpdated)){
                $eventsUpdated = implode(',',$eventsUpdated);
                $logData=['type'=>'cron','domain'=>"Stubhub:zones","message"=>"(".$eventsUpdated.") Events zone updated successfully."];
                //Logs::create($logData);
            }
        }else{
            $logData=['type'=>'cron','domain'=>"Stubhub:zones","message"=>"No Events available to update zones"];
            //Logs::create($logData);
            Log::info("No Events available to update zones");
        }
    }
}
?>